# Generated by Django 3.2.18 on 2023-05-26 20:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('NEMO_rs2_access', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Cardholder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('cardholder_id', models.CharField(max_length=250, unique=True)),
                ('cardholder_name', models.CharField(blank=True, max_length=250, null=True)),
                ('key_name', models.CharField(max_length=250)),
                ('key_value', models.CharField(max_length=250)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_updated', models.DateTimeField(auto_now=True)),
            ],
        ),
    ]
