# Generated by Django 3.2.18 on 2023-05-18 21:57

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('NEMO', '0045_version_4_5_0'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserPreferencesDefaultProject',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('default_project', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='NEMO.project')),
                ('user_preferences', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='default_project', to='NEMO.userpreferences')),
            ],
            options={
                'verbose_name': 'Default project',
            },
        ),
        migrations.CreateModel(
            name='Reader',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reader_id', models.CharField(max_length=250, unique=True)),
                ('site_id', models.CharField(max_length=250)),
                ('reader_name', models.CharField(max_length=250)),
                ('installed', models.BooleanField()),
                ('data', models.JSONField()),
                ('reader_type', models.PositiveIntegerField(choices=[(1, 'Entrance'), (2, 'Exit')], null=True)),
                ('last_updated', models.DateTimeField(auto_now=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('area', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='NEMO.area')),
            ],
        ),
    ]
