# Generated by Django 4.2.16 on 2024-11-19 23:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_rs2_access", "0004_errorlog_event_data"),
    ]

    operations = [
        migrations.AlterField(
            model_name="errorlog",
            name="error_type",
            field=models.PositiveIntegerField(
                choices=[
                    (1, "General"),
                    (2, "No active projects"),
                    (3, "No matching user"),
                    (4, "Logout without record"),
                    (5, "Login attempt while already logged in"),
                ],
                default=1,
            ),
        ),
    ]
