# Generated by Django 3.2.20 on 2023-08-09 19:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('NEMO_rs2_access', '0002_cardholder'),
    ]

    operations = [
        migrations.CreateModel(
            name='ErrorLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('description', models.TextField()),
                ('error_type', models.PositiveIntegerField(choices=[(1, 'General'), (2, 'No active projects'), (3, 'No matching user')], default=1)),
                ('data', models.CharField(blank=True, max_length=255, null=True)),
            ],
            options = {"ordering": ["-created"]}
        ),
    ]
