# Generated by Django 3.2.20 on 2023-08-10 18:18

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('NEMO_rs2_access', '0003_errorlog'),
    ]

    operations = [
        migrations.RenameField(
            model_name='errorlog',
            old_name='data',
            new_name='cardholder_value',
        ),
        migrations.AddField(
            model_name='errorlog',
            name='event_data',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='errorlog',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='errorlog',
            name='error_type',
            field=models.PositiveIntegerField(choices=[(1, 'General'), (2, 'No active projects'), (3, 'No matching user'), (4, 'Logout without record')], default=1),
        ),
    ]
